@@ -17,11 +17,11 @@ import (
 	"k8s.io/client-go/kubernetes"
 	_ "k8s.io/client-go/plugin/pkg/client/auth"
 
-	"github.com/open-cluster-management/config-policy-controller/pkg/apis"
-	"github.com/open-cluster-management/config-policy-controller/pkg/common"
-	"github.com/open-cluster-management/config-policy-controller/pkg/controller"
-	policyStatusHandler "github.com/open-cluster-management/config-policy-controller/pkg/controller/configurationpolicy"
-	"github.com/open-cluster-management/config-policy-controller/version"
+	"github.com/stolostron/config-policy-controller/pkg/apis"
+	"github.com/stolostron/config-policy-controller/pkg/common"
+	"github.com/stolostron/config-policy-controller/pkg/controller"
+	policyStatusHandler "github.com/stolostron/config-policy-controller/pkg/controller/configurationpolicy"
+	"github.com/stolostron/config-policy-controller/version"
 
 	"github.com/operator-framework/operator-sdk/pkg/k8sutil"
 	"github.com/operator-framework/operator-sdk/pkg/leader"
