@@ -11,7 +11,7 @@ import (
 	"testing"
 
 	"github.com/onsi/gomega"
-	"github.com/open-cluster-management/config-policy-controller/pkg/apis"
+	"github.com/stolostron/config-policy-controller/pkg/apis"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
 	"sigs.k8s.io/controller-runtime/pkg/envtest"
