@@ -1,8 +1,11 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package common
 
 import (
+	"context"
+
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
 
@@ -34,7 +37,7 @@ func GetSelectedNamespaces(included, excluded, allNamespaces []string) []string
 //GetAllNamespaces gets the list of all namespaces from k8s
 func GetAllNamespaces() (list []string, err error) {
 	namespaces := (*KubeClient).CoreV1().Namespaces()
-	namespaceList, err := namespaces.List(metav1.ListOptions{})
+	namespaceList, err := namespaces.List(context.TODO(), metav1.ListOptions{})
 
 	namespacesNames := []string{}
 	for _, n := range namespaceList.Items {
